#include <stdio.h>
#include <cmath>
#include <algorithm>

inline double sqr(double x) {
  return x*x;
}

class Point {
  public:
    double x,y,z;
    Point(){};
    Point(double x,double y,double z):x(x),y(y),z(z){};
    Point& operator=(const Point& rhs) {
      this->x = rhs.x;
      this->y = rhs.y;
      this->z = rhs.z;
      return *this;
    }
};

Point operator+(const Point a, const Point b) {
  return Point(a.x+b.x, a.y+b.y, a.z+b.z);
}

Point operator*(const Point p, double a) {
  return Point(a*p.x, a*p.y, a*p.z);
}

double distance(const Point& a, const Point& b) {
  return sqrt(sqr(a.x-b.x) + sqr(a.y-b.y) + sqr(a.z-b.z));
}

#define MAXN 200
Point coord[MAXN];
double radius[MAXN];
int id[MAXN];
int n;
double W,H,D;

#define FOR(q,n) for (int q=0; q<n; q++)

void read_input() {
  scanf("%lf %lf %lf", &W, &H, &D);
  int classes;
  scanf("%d", &classes);
  int j = 0;
  FOR(i, classes) {
    int cnt;
    double r;
    scanf("%d %lf", &cnt, &r);
    FOR(ii, cnt) {
      radius[j] = r;
      id[j] = i+1;
      j++;
    }
  }
  n=j;
}

const double EPS = 1e-5;
bool is_ok() {
  FOR(i, n) {
    Point& p = coord[i];
    double r = radius[i];
    if (p.x < r + EPS || p.x + r + EPS > W) return false;
    if (p.y < r + EPS || p.y + r + EPS > H) return false;
    if (p.z < r + EPS || p.z + r + EPS > D) return false;
  }
  FOR(i, n) {
    FOR(j, i) {
      if (distance(coord[i], coord[j]) < radius[i] + radius[j] + EPS) return false;
    }
  }
  return true;
}

void write_output() {
  FOR(i,n) {
    printf("%d %.9lf %.9lf %.9lf\n", id[i], coord[i].x, coord[i].y, coord[i].z);
  }
}

double _random(double min, double max) {
  const int T = 10000;
  return min + (rand() % T) * 1.0 / T * (max-min);
}

void random_solve() {
  FOR(i,n) coord[i].x = _random(0,W);
  FOR(i,n) coord[i].y = _random(0,H);
  FOR(i,n) coord[i].z = _random(0,D);

  const int MAXITER = 10000;

  FOR(iter, MAXITER) {
    double badness = 0;
    FOR(i, n) {
      Point& p = coord[i];
      double r = radius[i];
      if (p.x < r + EPS) {
        badness = std::max(badness, r - p.x);
        p.x = r + _random(1,2)*EPS;
      }
      if (p.y < r + EPS) {
        badness = std::max(badness, r - p.y);
        p.y = r + _random(1,2)*EPS;
      }
      if (p.z < r + EPS) {
        badness = std::max(badness, r - p.z);
        p.z = r + _random(1,2)*EPS;
      }
      if (p.x + r + EPS > W) {
        badness = std::max(badness, p.x + r - W);
        p.x = W - r - _random(1,2)*EPS;
      }
      if (p.y + r + EPS > H) {
        badness = std::max(badness, p.y + r - H);
        p.y = H - r - _random(1,2)*EPS;
      }
      if (p.z + r + EPS > D) {
        badness = std::max(badness, p.z + r - D);
        p.z = D - r - _random(1,2)*EPS;
      }
    }
    FOR(i, n) {
      FOR(j,i) {
        Point& a = coord[i];
        Point& b = coord[j];
        double r = radius[i] + radius[j];
        if (distance(a,b) < r + EPS) {
          badness = std::max(badness, r - distance(a,b));
          double scale = (r + _random(1,2)*EPS) / (1e-12 + distance(a,b));
          Point mid = a*0.5+b*0.5;
          a = (a+(mid*-1)) * scale + mid;
          b = (b+(mid*-1)) * scale + mid;
        }
      }
    }
    if (is_ok()) break;
  }
}

int main() {
  int t;
  scanf("%d", &t);
  FOR(q, t) {
    read_input();
    while (!is_ok()) {random_solve();}
    write_output();
  }
}
